# parameters to be changed in each run
file.site <- "DHMM.site"
file.output <- "DHMM.output"
states <- 3
randomStart <- 100
maxIteration <- 100


## R script ##
totype <- function(resource) {
	types <- c()
	if(max(resource[,4:7]) > 1) {
		mutation_type <- c("estimated", "real")[2]
		resource[,4:7] <- floor(resource[,4:7])
	} else if(sum(resource[,4:7] - floor(resource[,4:7])) > 0) {
		mutation_type <- c("estimated", "real")[1]
	}

	types2 <- matrix(NA, nrow=0, ncol=ncol(resource))
	site <- matrix(0, nrow=nrow(resource), ncol=2)
	for( i in 1:nrow(resource)) {
		if(sum(resource[i,]) == 0) {
			site[i,] <- c(i, -1)
			next
		}
		type <- paste(resource[i,], collapse=",")
		type_id <- which(types == type)
		if(length(type_id) <= 0) {
			types2 <- rbind(types2, resource[i,])
			types[length(types)+1] <- type
			type_id <- length(types)
		}
		site[i,] <- c(i, type_id)
	}
	snp <- resource[site[,2] != -1,]
	site <- site[site[,2] != -1, ]
	return(list(site=site, type=types2, snp=snp))
}

initialModel <- function(n=2, sites) {
	if(n == 1) {
		snp_mean_freq <- apply(sites$snp[,4:7], 2, sum)/c(apply(sites$snp[,1:3], 2, sum),nrow(sites$snp))
		if(snp_mean_freq[4] == 0) {
			snp_mean_freq <- snp_mean_freq[1:3]
		}
		emission <- snp_mean_freq
		transition <- 1
		initial <- 1
		return(list(transition=matrix(transition,nrow=1), emission=matrix(emission,nrow=1), initial=initial, LL=-Inf, AIC=Inf, BIC=Inf))
	} else {
		snp_mean_freq <- apply(sites$snp[,4:7], 2, sum)/c(apply(sites$snp[,1:3], 2, sum),nrow(sites$snp))
		if(snp_mean_freq[4] == 0) {
			snp_mean_freq <- snp_mean_freq[1:3]
		}
		emission <- sapply(log(snp_mean_freq), function(x) exp(rnorm(n, mean=x, sd=abs(x)/2)))
		if(mutation_type == "estimated") {
			emission[emission>=1] <- 0.99
		}
		transition <- matrix(exp(runif(n=n*n, min=log(0.0001), max=log(0.99/n))),ncol=n)
		transition[transition > 0.1] <- 0.1
		diag(transition) <- 1 - apply(transition, 1, sum) + diag(transition)
		initial <- rgamma(n, 1)
		return(list(transition=transition, emission=emission, initial=initial/sum(initial), LL=-Inf, AIC=Inf, BIC=Inf, diff=Inf))
	}
}
readModel <- function (modelfile) {
	fin <- file(modelfile, "r")
	no.state <- 0
	read <- "None"
	while (length(oneLine <- readLines(fin, n = 1, warn = FALSE)) > 0) {
		if(oneLine == "Transition:") {
			read <- "Transition"
			transition <- c()
			emission <- c()
			initial <- c()
			next
		} else if (oneLine == "Emission:") {
			read <- "Emission"
			next
		} else if (oneLine == "Initial:") {
			read <- "Initial"
			next
		} else if (oneLine == "LL, Diff, AIC, BIC:") {
			read <- "None"
			next
		}
		
		if (read != "None") {
			tmp <- as.numeric(strsplit(oneLine, " ")[[1]])
			if(read == "Transition") {
				no.state <- length(tmp)
				transition <- c(transition, tmp)
			}
			else if (read == "Emission") {
				emission <- c(emission, tmp)
			} else {
				initial <- tmp
			}
		}
	}
	close(fin)
	states <- no.state
	return(list(transition=matrix(byrow=TRUE, nrow=no.state, transition), emission=matrix(byrow=TRUE, nrow=no.state, emission), initial=initial, LL=-Inf, AIC=Inf, BIC=Inf, diff=Inf))
}


lkEstimate <- function(emission, sites) {
	if(is.matrix(emission) == FALSE) {
		emission <- matrix(emission, ncol = length(emission))
	}
	logLk <- matrix(0, ncol=nrow(emission), nrow=nrow(sites$type))
	for( i in 1:nrow(emission)) {
		if(ncol(emission) == 4) {
			if(mutation_type == "estimated") {
				logLk[,i] <- apply(sites$type, 1, function(x) (log((emission[i,1]*x[1])^x[4]*(1-emission[i,1]*x[1])^(1-x[4])) +
															   log((emission[i,2]*x[2])^x[5]*(1-emission[i,2]*x[2])^(1-x[5])) +
															   log((emission[i,3]*x[3])^x[6]*(1-emission[i,3]*x[3])^(1-x[6])) +
															   log((emission[i,4])^x[7]*(1-emission[i,4])^(1-x[7])) ))
			} else if (mutation_type == "real") {
				logLk[,i] <- apply(sites$type, 1, function(x) (dpois(x[4],emission[i,1]*x[1],TRUE)+dpois(x[5],emission[i,2]*x[2],TRUE)+dpois(x[6],emission[i,3]*x[3],TRUE)+dpois(x[7],emission[i,4],TRUE)))
			}
		} else {
			if(mutation_type == "estimated") {
				logLk[,i] <- apply(sites$type, 1, function(x) (log((emission[i,1]*x[1])^x[4]*(1-emission[i,1]*x[1])^(1-x[4])) +
															   log((emission[i,2]*x[2])^x[5]*(1-emission[i,2]*x[2])^(1-x[5])) +
															   log((emission[i,3]*x[3])^x[6]*(1-emission[i,3]*x[3])^(1-x[6])) ))
			} else if (mutation_type == "real") {
				logLk[,i] <- apply(sites$type, 1, function(x) (dpois(x[4],emission[i,1]*x[1],TRUE) + dpois(x[5],emission[i,2]*x[2],TRUE) + dpois(x[6],emission[i,3]*x[3],TRUE)))
			}

		}
	}
	prob <- t(sapply(sites$site[,2], function(x) logLk[x, ]))
	return(prob)
}


forwardback <- function(prob, Pi, delta, fortran=FALSE){
    m <- nrow(Pi)
    n <- nrow(prob)
	prob <- exp(prob)
    #   forward probabilities alpha_ij
    phi <- as.double(delta)
    logalpha <- matrix(as.double(rep(0, m*n)), nrow=n)
    lscale <- as.double(0)

	for (i in 1:n){
		if (i > 1) phi <- phi %*% Pi
		phi <- phi*prob[i,]
		sumphi <- sum(phi)
		phi <- phi/sumphi
		lscale <- lscale + log(sumphi)
		logalpha[i,] <- log(phi) + lscale
	}
	LL <- lscale

	logbeta <- matrix(as.double(rep(0, m*n)), nrow=n)
    phi <- as.double(rep(1/m, m))
    lscale <- as.double(log(m))

	for (i in seq(n-1, 1, -1)){
		phi <- Pi %*% (prob[i+1,]*phi)
		logbeta[i,] <- log(phi) + lscale
		sumphi <- sum(phi)
		phi <- phi/sumphi
		lscale <- lscale + log(sumphi)
	}
    return(list(logalpha=logalpha, logbeta=logbeta, LL=LL))
}

BaumWelch <- function (model, sites){
    Pi <- model$transition
    delta <- model$initial
	pm <- model$emission
	oldLL <- -Inf
	
	if(nrow(pm) == 1) {
		prob <- lkEstimate(pm, sites)
		model$LL <- sum(prob)
		model$AIC <- -2*model$LL + 2*(length(pm))
		model$BIC <- -2*model$LL + log(length(prob))*(length(pm))
		cat("iter = 1; LL = ", model$LL, "; diff = ", Inf, "; AIC = ", model$AIC, "; BIC = ", model$BIC, "\n")
		return(model)
	}
	
	for ( iter in 1:maxIteration) {
		prob <- lkEstimate(pm, sites)
		cond <- Estep(prob, Pi, delta)
		
		diff <- cond$LL - oldLL
		AIC <- -2*cond$LL + 2*(nrow(Pi)*(nrow(Pi)-1) + ncol(pm)*nrow(pm))
		BIC <- -2*cond$LL + log(nrow(prob))*(nrow(Pi)*(nrow(Pi)-1) + ncol(pm)*nrow(pm))
		cat("iter = ", iter, "; LL = ", cond$LL, "; diff = ", diff, "; AIC = ", AIC, "; BIC = ", BIC, "\n")
		print(Pi)
		print(pm)
		if(diff < 0.00001) break
        #----  Mstep  ----
        #Pi <- diag(1/apply(cond$v, MARGIN = 2, FUN = sum)) %*% apply(cond$v, MARGIN = c(2, 3), FUN = sum)
		Pi <- cond$Pi
        delta <- cond$u[1, ]

		if(ncol(pm) == 4) {
			pm <- t(apply(cond$u, 2, function(ux) apply(sites$snp[,4:7]*ux, 2, sum)))/t(apply(cond$u, 2, function(ux) apply(cbind(sites$snp[,1:3], 1)*ux, 2, sum)))
		} else {
			pm <- t(apply(cond$u, 2, function(ux) apply(sites$snp[,4:6]*ux, 2, sum)))/t(apply(cond$u, 2, function(ux) apply(sites$snp[,1:3]*ux, 2, sum)))
		}
        oldLL <- cond$LL
    }
    model$initial <- delta
    model$transition <- Pi
    model$emission <- pm
    model$LL <- cond$LL
	model$AIC <- AIC
	model$BIC <- BIC
	model$diff <- diff
    return(model)
}

Estep <- function (prob, Pi, delta) {
    m <- nrow(Pi)
    n <- nrow(prob)
    y <- forwardback(prob, Pi, delta)
    LL <- y$LL
    v <- array(NA, dim = c(n - 1, m, m))
    for (k in 1:m) {
        logPi <- matrix(log(Pi[, k]), byrow = TRUE, nrow = n - 1, ncol = m)
        logPbeta <- matrix(prob[-1, k] + y$logbeta[-1, k], byrow = FALSE, nrow = n - 1, ncol = m)
        v[, , k] <- logPi + y$logalpha[-n, ] + logPbeta - LL
    }
    v <- exp(v)
	Pi <- diag(1/apply(v, MARGIN = 2, FUN = sum)) %*% apply(v, MARGIN = c(2, 3), FUN = sum)
	rm(v)
	gc()
	u <- exp(y$logalpha + y$logbeta - LL)
    return(list(u = u, Pi = Pi, LL = LL))
}
statePerSite <- function (prob, Pi, delta) {
    m <- nrow(Pi)
    n <- nrow(prob)
    y <- forwardback(prob, Pi, delta)
    LL <- y$LL
	u <- exp(y$logalpha + y$logbeta - LL)
    return(list(u = u))
}
estimateState <- function (model, sites){
    Pi <- model$transition
    delta <- model$initial
	pm <- model$emission
	oldLL <- -Inf
	if(maxIteration > 0) {
		for ( iter in 1:maxIteration) {
			prob <- lkEstimate(pm, sites)
			cond <- statePerSite(prob, Pi, delta)
			return (cond$u)
		}
	}
}
writeSites <- function (fout, sites, persite) {
	
	col.snp <- ncol(sites$snp)
	no.state <- ncol(persite)
	no.sites <- sites$site[nrow(sites$site),]
	result <- matrix(0, ncol = col.snp+no.state, nrow=no.sites)
	result[sites$site[,1],] <- cbind(sites$snp, persite)

	write(t(result), file=fout, ncol=ncol(result))
}


options(echo=FALSE) # if you want see commands in output file
args <- commandArgs(trailingOnly = TRUE)
options(digits=15)

if(length(args) >= 3) {
	file.site <- args[1]
	file.output <- args[2]
	states <- args[3]
	if(length(args) >= 4) { randomStart <- args[4] }
	if(length(args) >= 5) { maxIteration <- args[5] }
} else {
	print("DHMM.R <site file> <output file> <no. state / model file> <no. of random starts (0 for sites only) > <no. of iterations>\n")
}

mutation_type <- c("estimated", "real")[2]

resource <- matrix(scan(file.site, what=double(0), n=99999999), byrow=TRUE, ncol=7)
sites <- totype(resource)

if( length(grep("^[0-9\\.]+$", states, perl=TRUE)) > 0 ) {
	states <- as.numeric(states)
	minmodel <- initialModel(states, sites)
} else {
	minmodel <- readModel(states)
}

fout <- file(file.output, "w")
open(fout)
if(maxIteration > 0) {
	minmodel <- BaumWelch(minmodel, sites)
	write("Transition:", file=fout)
	write(t(minmodel$transition), file=fout, ncol=ncol(minmodel$transition))
	write("Emission:", file=fout)
	write(t(minmodel$emission), file=fout, ncol=ncol(minmodel$emission))
	write("Initial:", file=fout)
	write(t(minmodel$initial), file=fout, ncol=ncol(minmodel$transition))
	write("LL, Diff, AIC, BIC:", file=fout)
	write(t(c(minmodel$LL, minmodel$diff, minmodel$AIC, minmodel$BIC)), file=fout, ncol = 4)
	if(randomStart > 1 && states > 1) {
		for ( i in 2:randomStart) {
			cat("Random start: ", i, "\n")
			model <- initialModel(states, sites)
			model <- BaumWelch(model, sites)
			if(model$LL > minmodel$LL) {
				minmodel <- model

				write("Transition:", file=fout)
				write(t(minmodel$transition), file=fout, ncol=ncol(minmodel$transition))
				write("Emission:", file=fout)
				write(t(minmodel$emission), file=fout, ncol=ncol(minmodel$emission))
				write("Initial:", file=fout)
				write(t(minmodel$initial), file=fout, ncol=ncol(minmodel$transition))
				write("LL, Diff, AIC, BIC:", file=fout)
				write(t(c(minmodel$LL, minmodel$diff, minmodel$AIC, minmodel$BIC)), file=fout, ncol = 4)
			}
		}
	}
}
if(states > 1) {
	persite <- estimateState(minmodel, sites)
	writeSites(fout,sites, persite)
}
close(fout)
