quickAnc <- function (tip.state, edges, rate=1, label=NULL, type="FB") {
	if(length(label) > 0) {
		x <- tip.state[label,]
	} else {
		x <- tip.state
	}
	edges[edges[,3]==0,3] <- 0.0000001
	
	if(is.null(dim(edges)) || dim(edges)[1] <= 1) {
		nb.node <- max(nrow(x), edges[1:2])
		alpha <- matrix(0, nb.node, ncol=ncol(x))
		colnames(alpha) <- colnames(x)
		for ( i in 1:length(x)) {
			alpha[i,] <- x[i,]
		}
		alpha[edges[1],] <- x[edges[2],]
		return(alpha)
	}

	nb.node <- max(nrow(x), edges[,1:2])
	nb.branch <- nrow(edges)
	nb.tip <- nrow(x)
	nb.type <- ncol(x)

	if(is.null(dim(x)) || nb.type == 1) {
		return (matrix(1, nb.node, ncol=1))
	}
	
	dif <- matrix(0, nrow=nb.branch, ncol=nb.type)
	alpha <- matrix(1/nb.type, nb.node, ncol=nb.type)
	colnames(alpha) <- colnames(x)
	for ( i in 1:nb.tip) {
		alpha[i,] <- x[i,]
	}

	E <- ape::matexpo
	Q <- matrix(rate, nb.type, nb.type)
	diag(Q) <- -(rowSums(Q) - diag(Q))
	
	for (i in seq(from = nb.branch, by = -1, to = 1)) {
		if(sum(alpha[edges[i,2], ]) > 0) {
			dif[i,] <- E(Q * edges[i,3]) %*% alpha[edges[i,2], ]
			v <- alpha[edges[i,1], ] * dif[i,]
			alpha[edges[i,1], ] <- v / sum(v)
		}
	}
	if(type == "FB") {
		for ( i in 1:nb.branch) {
			if(sum(dif[i,]) > 0) {
				rev_dif <- E(Q * edges[i, 3]) %*% (alpha[edges[i,1], ] / dif[i,])
				v <- alpha[edges[i,2], ] * rev_dif
			} else {
				v <- E(Q * edges[i, 3]) %*% alpha[edges[i,1], ]
			}
			alpha[edges[i,2], ] <- v/sum(v)
		}
	} else if(type == "MAP") {
		tmp <- which.max(alpha[edges[1,1],])
		alpha[edges[1,1],] <- 0
		alpha[edges[1,1],tmp] <- 1
		
		for ( i in 1:nb.branch ) {
			rev_dif <- E(Q * edges[i, 3]) %*% (alpha[edges[i,1], ])
			if (sum(alpha[edges[i,2], ]) > 0) {
				v <- alpha[edges[i,2], ] * rev_dif
			} else {
				v <- rev_dif
			}
			alpha[edges[i,2], ] <- 0
			alpha[edges[i,2], which.max(v)] <- 1
		}
	} else if (type =="Gibbs") {
		r <- runif(1)*sum(alpha[edges[1,1],])
		for ( i in 1:no.type) {
			r <- r - alpha[edges[1,1],i]
			if( r <= 0) {
				alpha[edges[1,1],] = 0
				alpha[edges[1,1],i] <- 1
				break
			}
		}
	
		for ( i in 1:nb.branch) {
			rev_dif <- E(Q * edges[i, 3]) %*% (alpha[edges[i,1], ])
			if (sum(alpha[edges[i,2], ]) > 0) {
				v <- alpha[edges[i,2], ] * rev_dif
			} else {
				v <- rev_dif
			}
			r <- runif(1)*sum(v)
			for ( i in 1:no.type) {
				r <- r - v[i]
				if ( r <= 0) {
					alpha[edges[i,2], ] <- 0
					alpha[edges[i,2], i] <- 1
				}
			}
		}
	}
	return(alpha)
}

typeToAnc <- function (x, edges, rate, label=NULL, type="FB") {
	t <- as.character(unique(x))
	t <- t[t != "-"]
	t <- t[t != "n"]
	t <- t[t != "0"]
	mat <- matrix(0, nrow=length(x), ncol=length(t))
	colnames(mat) <- t
	rownames(mat) <- names(x)
	for ( i in 1:nrow(mat)) {
		if(x[i] != "-" && x[i] != "n" && as.character(x[i]) != "0") {
			mat[i, as.character(x[i])] <- 1
		}
	}
	return(quickAnc(mat, edges, rate, label, type))
}

rec.density <- 0.05
file.rec <- ""
encode <- c('A', 'C', 'G', 'T')

options(echo=FALSE) # if you want see commands in output file
args <- commandArgs(trailingOnly = TRUE)
if(length(args) < 3) {
        cat("Rscript RHMM_snp_allocate.R <file.SNP> <file.tree> <branch.rate> <file.rec> <rec.density>\n")
        quit()
} else {
        if(length(args) >= 1) {file.SNP <- args[1]}
        if(length(args) >= 2) {file.tree <- args[2]}
		if(length(args) >= 3) {branch.rate <- as.numeric(args[3])}
        if(length(args) >= 4) {file.rec <- args[4]}
		if(length(args) >= 5) {rec.density <- as.numeric(args[5])}
}
library(ape)
convert <- function (base=base) {
	code <- rep(0, length(base))
	code[which(base == "a")] <- 1
	code[which(base == "c")] <- 2
	code[which(base == "g")] <- 3
	code[which(base == "t")] <- 4
	code[which(base == "A")] <- 1
	code[which(base == "C")] <- 2
	code[which(base == "G")] <- 3
	code[which(base == "T")] <- 4
	return(code)
}

readSNP <- function (file.SNP) {
	fin  <- readLines(file.SNP, n=9999999, warn = FALSE)
	lines <- strsplit(fin, "\t")
	heads <- lines[[1]]
	sites <- c()
	snp <- matrix(0, nrow=length(fin)-1, ncol=length(heads)-1)
	for( i in 2:length(fin)) {
		sites[length(sites)+1] <- lines[[i]][1]
		snp[i-1,] <- convert(lines[[i]][2:length(heads)])
	}
	# snp <- matrix(byrow=TRUE, nrow=length(fin)-1, ncol=length(heads)-1, snp)
	colnames(snp) <- heads[2:length(heads)]
	rownames(snp) <- sites

	return(snp)
}

readREC <- function (file.rec, name=NULL) {
	mat <- matrix(scan(file.rec, n=9999999, what=character(0)), byrow=TRUE, ncol=5)
	region <- matrix(as.numeric(mat[,1:2]), ncol=2)
	type <- strsplit(mat[,5],"")
	types <- c()
	for ( i in 1:nrow(region)) {
		types <- c(types, type[[i]])
	}
	types <- matrix(byrow=TRUE, nrow= nrow(region), ncol=length(type[[1]]), types)
	colnames(types) <- name
	return(list(region=region, types=types))
}

readTREE <- function (file.tree, name=NULL) {
	if(is.null(name)) {
		name <- tree$tip.label
	}
	tree <- read.tree(file.tree)
	edges <- cbind(tree$edge, tree$edge.length)
	edges[edges[,3]==0, 3] <- 0.0000001
	bipartition <- c()
	for( i in 1:max(tree$edge)) {
		if(i <= length(tree$tip.label)) {
			id <- which(name == tree$tip.label[i])
			bipartition[i] <- paste(c(rep("B", id-1), "A", rep("B", length(tree$tip.label) - id)), collapse="")
		} else {
			result <- iteNode(edges, i)
			bipartition[i] <- paste(rep("B", length(tree$tip.label)), collapse="")
			for( r in result) {
				id <- which(name == tree$tip.label[r])
				substr(bipartition[i], id, id) <- "A";
			}
		}
	}
	return(list(edges=edges, label=tree$tip.label, tree=tree, bipartition=bipartition))
}
iteNode <- function (edges, id) {
	descend <- edges[edges[,1] == id, 2]
	if(length(descend) == 0) {
		return(id)
	} else {
		data <- c()
		for ( d in descend) {
			data <- c(data, iteNode(edges, d))
		}
		return(data)
	}
}

snp <- readSNP(file.SNP)
tree <- readTREE(file.tree, colnames(snp))

if(file.rec != "") {
	rec.region <- readREC(file.rec, colnames(snp))
}

for( i in 1:nrow(snp)) {
	site = as.numeric(rownames(snp)[i])
	mut.rate <- rep(1, nrow(tree$edges))
	if (file.rec != "") {
		recs <- which(rec.region$region[,1]<= site & rec.region$region[,2]>=site)
		if(file.rec != "") {
			for (id in recs) {
				states <- typeToAnc(rec.region$types[id,], tree$edges, rate=branch.rate, label=tree$label)
				mut.rate <- (1 - apply(tree$edges, 1, function(x) sum(abs(states[x[1],] - states[x[2],]))/2))*mut.rate
			}
		}
	}
	edges2 <- tree$edges
	edges2[,3] <- (mut.rate*edges2[,3] + (1-mut.rate)*rec.density/branch.rate)
	
	states <- typeToAnc(snp[i,], edges2, rate=branch.rate, label=tree$label, type="MAP")
	event <- rep('', nrow(tree$edges))
	for (j in 1:nrow(tree$edges)) {
		if (sum(abs(states[tree$edges[j,1],] - states[tree$edges[j,2],])) > 0) {
			event[j] <- paste (encode[as.numeric(colnames(states)[states[tree$edges[j,1],] > 0])],'->',encode[as.numeric(colnames(states)[states[tree$edges[j,2],] > 0])],sep='')
		}
	}
	# event <- apply(tree$edges, 1, function(x) sum(abs(states[x[1],] - states[x[2],]))/2)
	br <- cbind(tree$edges[event != '',2], event[event != ''])
	if(nrow(br) > 0) {
		cat(apply(br, 1, function(x) paste(c(site, paste("Br_", x[1], sep=""), tree$bipartition[as.numeric(x[1])], x[2]), collapse=" ")), sep="\n")
	}
}
